
#include "matrix.h"

// #include <cstdlib> // function exit
#include <stdexcept> // function exit

using namespace std;

string line; // current character
int pos; // character position
int len; // line length

char ch; // current character

void init ()
{
   line = "";
   pos = 0;
   len = 0;
   ch = ' ';
}

const bool trace = false;

void error (string msg)
{
   cerr << "Error: " << msg << endl;
   // exit (0);
   throw runtime_error (msg);
}

void readChar (istream & stream)
{
   while (pos >= len && stream.good ())
   {
      getline (stream, line);
      if (trace)
         cout << "Line " << line << endl;
      pos = 0;
      len = line.length ();
   }
   ch = line [pos];
   pos ++;

   if (trace)
   {
      cout << "Character ";
      if (ch >= ' ')
         cout << "'" << ch << "'";
      else
         cout << "chr (" << int (ch) << ")";
      cout << endl;
   }

   if (! stream.good ())
      error ("Cannot read character");
}

void skip (istream & stream)
{
   while (ch <= ' ')
      readChar (stream);
}

void check (istream & stream, char t)
{
   skip (stream);
   if (ch != t)
      error (string ("Character '") + t + "' expected");
   readChar (stream); // read next character
}

void check_last (istream & stream, char t)
{
   skip (stream);
   if (ch != t)
      error (string ("Character '") + t + "' expected");
}

void readNumber (istream & stream, ElementType & number)
{
   bool sign = 1;
   ElementType value = 0;

   skip (stream);
   if (ch == '-')
   {
      sign = -1;
      readChar (stream); // skip minus
      skip (stream);
   }

   if (ch < '0' || ch > '9')
      error ("Digit expected");

   while (ch >= '0' && ch <= '9')
   {
      value = 10 * value + (ch - '0');
      readChar (stream); // skip digit
   }

   if (ch == '.')
   {
      readChar (stream); // skip dot

      if (ch < '0' || ch > '9')
         error ("Digit expected");

      ElementType scale = 1;
      while (ch >= '0' && ch <= '9')
      {
         scale = scale / 10;
         value = value + (ch - '0') * scale;
         readChar (stream); // skip digit
      }
   }

   number = sign * value;
   if (trace)
      cout << "Number " << number << endl;
}

void readVector (istream & stream, VectorType & vector)
{
   init ();
   readChar (stream); // read first character

   check (stream, '(');
   for (int i = 0; i < VectorSize; i++)
   {
      readNumber (stream, vector [i]);
      if (i < VectorSize - 1)
         check (stream, ',');
   }
   check_last (stream, ')');
}

void readMatrix (istream & stream, MatrixType & matrix)
{
   init ();
   readChar (stream); // read first character

   check (stream, '[');
   for (int i = 0; i < MatrixSize; i++)
   {
      check (stream, '(');
      for (int j = 0; j < MatrixSize; j++)
      {
         readNumber (stream, matrix [i][j]);
         if (j < MatrixSize - 1)
            check (stream, ',');
      }
      check (stream, ')');
      if (i < MatrixSize - 1)
         check (stream, ',');
   }
   check_last (stream, ']');
}

void writeVector (ostream & stream, VectorType & vector)
{
   stream << "(";
   for (int i = 0; i < VectorSize; i++)
   {
      stream << vector [i];
      if (i < VectorSize - 1)
         stream << ", ";
   }
   stream << ")" << endl;
}

void writeMatrix (ostream & stream, MatrixType & matrix)
{
   stream << "[" << endl;
   for (int i = 0; i < MatrixSize; i++)
   {
      stream << "    ";
      stream << "(";
      for (int j = 0; j < MatrixSize; j++)
      {
         stream << matrix [i][j];
         if (j < MatrixSize - 1)
            stream << ", ";
      }
      stream << ")";
      if (i < MatrixSize - 1)
         stream << ",";
      stream << endl;
   }
   stream << "]" << endl;
}


